Option Explicit

Function Soundex (n As String) As Integer
'*****************************************************************
'*                                                               *
'*      Procedure Name:   Soundex                                *
'*                                                               *
'*             Created:   1/5/94     By: Isaac Rabinovitch       *
'*            Modified:              By:                         *
'*                                                               *
'*            Comments:  Purely numeric version of standard      *
'*                       standard Soundex function.  See         *
'*                       description below                       *
'*                                                               *
'*****************************************************************

'Western surnames that sound similar (at least in their first few
'syllables) tend to produce proximate Soundex codes.  Databases use
'Soundex codes to support name searches when the exact spelling
'is uncertain.
'
'Most Soundex functions produce 4-character strings, where the first 
'character is the first letter of the name and the remaining 3
'characters roughly describe the way the beginning of the name sounds.
'The basic idea is that if we ignore all vowels (we assume Y is a
'vowel) and the consonants W and H, we can divide the rest of the
'alphabet into 6 sound categories:
'
'   1: B, F, P, V
'   2: C, G, J, K, Q, S, X, Z
'   3: D, T
'   4: L
'   5: M, N
'   6: R
'
'Starting with the second letter, we examine each letter in turn, and
'if we can use it, we append it to our string, to a maximum of 4.
'We skip a consonant if it "sounds like" (has the same category number
'as) the previous consonant, unless there's a vowel between the two.
'
'Instead of the usual string, this function returns an integer.  1000-1666
'means the name starts with "A", 2000-2666 means "B", and so on.
'Non-alphabetic characters are ignored.  Null string (or string with no 
'non-alphabetic chars) returns 0

    Dim l As Integer, scode As Integer
    Dim ccode As Integer, prevcode As Integer
    Dim i As Integer, mp As Integer, c As String

    l = Len(n)
    If l = 0 Then
        Soundex = 0
        Exit Function
    End If
    n = UCase$(n)
    scode = 0    'code calculated so far
    i = 0        'number of characters processed
    mp = 3       'significance of current code digit
    Do While i < l And mp >= 0
        i = i + 1
        c = Mid(n, i, 1)
        If c >= "A" And c <= "Z" Then
            Select Case c
                Case "W", "H" 'simply skipped
                    ccode = -1
                Case "A", "E", "I", "O", "U", "Y" 'skipped, but "remembered"
                    ccode = 0
                Case "B", "F", "P", "V"
                    ccode = 1
                Case "C", "G", "J", "K", "Q", "S", "X", "Z"
                    ccode = 2
                Case "D", "T"
                    ccode = 3
                Case "L"
                    ccode = 4
                Case "M", "N"
                    ccode = 5
                Case "R"
                    ccode = 6
            End Select
            'first letter?  code it as "itself"
            If mp = 3 Then
                scode = 1000 * (Asc(c) - Asc("A") + 1)
                mp = 2
            'subsequent letters:  is it a codable consonant?  is it a new category?
            ElseIf ccode > 0 And ccode <> prevcode Then
                'insert the category digit into the most significant unused place
                scode = scode + 10 ^ mp * ccode
                mp = mp - 1
            End If
            'ignore w and h, remember other consonants (to detect repetition) and
            'vowels (they serve as consonant separators)
            If ccode >= 0 Then prevcode = ccode
        End If 'c >= "A" Or c <= "Z"
    Loop 'While i < l And mp >= 0
    Soundex = scode
End Function

